//
// Copyright (c) 2006 Regents of the SIGNET lab, University of Padova.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. Neither the name of the University of Padova (SIGNET lab) nor the 
//    names of its contributors may be used to endorse or promote products 
//    derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//


#include "sandbox-plugin1-clmsg.h"
#include "clsap.h"


extern ClMessage_t SANDBOXPLG1MSG;
extern ClMessage_t SANDBOXPLG1MSGS;

/*-------------------------------------------------------------------------------------------------------
	methods for ClMsgSandboxPlg1Disc class
---------------------------------------------------------------------------------------------------------*/ 

ClMsgSandboxPlg1Disc::ClMsgSandboxPlg1Disc() : ClMessage(SANDBOXPLG1MSGVERB, SANDBOXPLG1MSG) {}

ClMsgSandboxPlg1Disc::ClMsgSandboxPlg1Disc(ClMsgSandboxPlg1Disc *m) : ClMessage(m)
{
}

ClMsgSandboxPlg1Disc::ClMsgSandboxPlg1Disc(int verbosity, DestinationType dtype, int source, int value): ClMessage(verbosity, SANDBOXPLG1MSG, dtype, source, value)
{
}

// retrun a copy of the message
ClMessage *ClMsgSandboxPlg1Disc::copy()
{
	ClMessage *temp = new ClMsgSandboxPlg1Disc(this);
	return (temp);
}

/*-------------------------------------------------------------------------------------------------------
	methods for ClMsgSandboxPlg1DiscTracer class
---------------------------------------------------------------------------------------------------------*/ 

ClMsgSandboxPlg1DiscTracer::ClMsgSandboxPlg1DiscTracer() : ClMessageTracer(SANDBOXPLG1MSG)
{
}

void ClMsgSandboxPlg1DiscTracer::format(ClMessage *m, ConnectorTrace *sap)
{
	writeTrace(sap, " [SNBXPLG1: discover ]");
}

/*-------------------------------------------------------------------------------------------------------
	methods for ClMsgSandboxPlg1Set class
---------------------------------------------------------------------------------------------------------*/ 

ClMsgSandboxPlg1Set::ClMsgSandboxPlg1Set() : ClMessage(SANDBOXPLG1MSGVERB, SANDBOXPLG1MSGS) {}

ClMsgSandboxPlg1Set::ClMsgSandboxPlg1Set(ClMsgSandboxPlg1Set *m) : ClMessage(m)
{
}

ClMsgSandboxPlg1Set::ClMsgSandboxPlg1Set(int verbosity, DestinationType dtype, int source, int value, int newSize): ClMessage(verbosity, SANDBOXPLG1MSGS, dtype, source, value), newSize_(newSize)
{
}

// retrun a copy of the message
ClMessage *ClMsgSandboxPlg1Set::copy()
{
	ClMessage *temp = new ClMsgSandboxPlg1Set(this);
	((ClMsgSandboxPlg1Set*)temp)->newSize_ = this->newSize_;
	return (temp);
}

int ClMsgSandboxPlg1Set::getSize()
{
	return (newSize_);
}


/*-------------------------------------------------------------------------------------------------------
	methods for ClMsgSandboxFe1DiscTracer class
---------------------------------------------------------------------------------------------------------*/ 

ClMsgSandboxPlg1SetTracer::ClMsgSandboxPlg1SetTracer() : ClMessageTracer(SANDBOXPLG1MSGS)
{
}

void ClMsgSandboxPlg1SetTracer::format(ClMessage *m, ConnectorTrace *sap)
{
	writeTrace(sap, " [SNBXPLG1set: -newSize%d ]", ((ClMsgSandboxPlg1Set *)m)->getSize());
}

